clc
clear
% Setting path to allow interface/communication between matlab and Xpress:
addpath(fullfile(getenv('XPRESSDIR'),'/matlab'));


%----------------------------------------------------------------------
% INPUT DATA 
%----------------------------------------------------------------------

% Name of scenario to run
Instance = '90_turbines_instans_01';

% Med eller uten kjring? 
MoselRunning = true;            %Kjrer i mosel 
RunningOnce = true;             %Tester om mulig for n iterasjon

% Name of scenario file with saved variables: 
VariableFile = [Instance, '.mat']; 
% Load variables 
load(VariableFile); 
% Name of Xpress model 
modelname = 'Model_RHH2_150529.mos';

% Number of iterations to solve for: 
nIterations = 1;             
% Length of planning period:
forecastingPeriod = 1; 
nShifts = forecastingPeriod;

% Dummy variables for printing output files: 
taskDiff = 1000; 
endDepot = 2000; 



%----------------------------------------------------------------------
% PROCESSING INPUT DATA 
%----------------------------------------------------------------------
% Creating instance i = 1 based on scenario file and length of planning period: 

% Weather 
    for period = 1:forecastingPeriod
        Wind(period) = WindLongHorizon(period);
        LWindow(:,period) = StartWindow(:,period);
        UWindow(:,period) = EndWindow(:,period);
    end % for period

  
% Corrective tasks due to checked alarms in previous shifts: 
    iteration = 1; 
    NumberOfTasks = 0; 
    WindFarmTasks = WindFarmTasks_0;
    VesselsTasks = VesselsTasks_0;
    Type = Type_0; 
    TTask = TTask_0;
    TechTask = TechTask_0; 
    nCorrTasks = nCorrTasks_0; 
    nTasks = nTasks_0; 
    TasksWindFarm = TasksWindFarm_0;
    ExtraOrdinaryTask = ExtraOrdinaryTask_0;
    AlarmResultingInTask = AlarmResultingInTask_0; 
    NumberOfTasks = NumberOfTasks + nTasks; 

% Triggered alarms for the planning period:
    for period = 1:1
        NewWindFarmTasks = eval(['WindFarmTasks_' num2str(period)]);          % Collecting new tasks from shift "period" and saving them in new vectors
        NewVesselsTasks = eval(['VesselsTasks_' num2str(period)]);
        NewType = eval(['Type_' num2str(period)]);
        NewTTask = eval(['TTask_' num2str(period)]);
        NewTechTask = eval(['TechTask_' num2str(period)]);
        NewnCorrTasks = eval(['nCorrTasks_' num2str(period)]);
        NewTasksWindFarm = eval(['TasksWindFarm_' num2str(period)]);
        NewExtraOrdinaryTask = eval(['ExtraOrdinaryTask_' num2str(period)]);
        NewAlarmResultingInTask = eval(['AlarmResultingInTask_' num2str(period)]);
        NewnTasks = NewnCorrTasks;

        for m = 1:(NewnTasks)
        TTask(m+NumberOfTasks) = NewTTask(m);                                  % Adding the new vectors to the excisting task vectors  
        VesselsTasks(m+NumberOfTasks,:) = NewVesselsTasks(m,:); 
        TechTask(m+NumberOfTasks) = NewTechTask(m);
        ExtraOrdinaryTask(m+NumberOfTasks) = NewExtraOrdinaryTask(m);
        Type(m+NumberOfTasks) = NewType(m);
        TasksWindFarm(m+NumberOfTasks) = NewTasksWindFarm(m);
        AlarmResultingInTask(m+NumberOfTasks) = NewAlarmResultingInTask(m);
        end % for m 

        nTasks = nTasks + NewnTasks;
        NumberOfTasks = NumberOfTasks + NewnTasks; 
        nCorrTasks = nCorrTasks + NewnCorrTasks;
        

        % WindFarmTasks(i,m) 
        A = size(WindFarmTasks,2);                      % Dummy variable with size of NewWindFarmTasks
        B = size(NewWindFarmTasks,2);                   % Dummy variable with size of WindFarmTasks

        for i = 1:B                                     % Adding zeros for all new tasks in WindFarmTasks
            WindFarmTasks(:,A+i) = 0;   
        end % for i 

        for i = 1:nWindFarms                            % Replacing zeros with the actual tasks 
        t = 1; 
            for m = 1:size(WindFarmTasks,2)
                if t <= B 
                    if WindFarmTasks(i,m) == 0
                        WindFarmTasks(i,m) = NewWindFarmTasks(i,t); 
                        t = t+1; 
                    end % if 
                end % if 
            end % for m 
        end % for i 

        C = size(WindFarmTasks,2)+1;                          % Dummy variable for the new size of WindFarmTasks
        for m = 1:C-1                                         % Deleting all excess columns in WindFarmTasks (colums where all rows = 0) 
            t=C-m;
            if WindFarmTasks(:,t) == 0
                WindFarmTasks(:,t) = []; 
            end % if 
        end % for m 
        
    
    end % period 


% Adding preventive tasks: 
    AmountPrevGenerated = 1.5;
    numberShiftsInSeason = round(365/numberOfSeasons);     

    % Number of preventive tasks to add and nPrevTasksWant for each wind farm: 
    for i = 1:nWindFarms 
        YearlyPreventiveHours(i) = YearlyPreventiveTasks(i)*TPrevTask;   
        HoursInshiftWant(i) = round(YearlyPreventiveHours(i)* AmountPrev(Season)*AmountPrevScheduled/numberShiftsInSeason);
        HoursInShift(i) = round(YearlyPreventiveHours(i)* AmountPrev(Season)*AmountPrevGenerated/numberShiftsInSeason);
        nPrevTasksWant(i) = round(HoursInshiftWant(i)/10); 
        nPrevTasks(i) = round(HoursInShift(i)/10);
    end % for i 
    nPrevTasks = sum(nPrevTasks);
    nPrevTasksWant = sum(nPrevTasksWant);

    if nPrevTasks == 0
        nPrevTasks = 1; 
    end 

    if nPrevTasksWant == 0
        nPrevTasksWant = 1; 
    end
    NumberOfTasks = NumberOfTasks + nPrevTasks;

    % Allocates preventive tasks to the wind farms
    Relation=[1,nWindFarms];
    nPrevtasksFarm =zeros(1, nWindFarms);
    for i=1:(nWindFarms-1)
        Relation(i)=nTurbines(i)/sum(nTurbines);
        nPrevTasksFarm(i)=floor(Relation(i)*nPrevTasks);
    end
    NumberAllocated=sum(nPrevTasksFarm(i));
    nPrevTasksFarm(nWindFarms)=nPrevTasks-NumberAllocated;

    % Giving properties to the preventive tasks 
    p=nCorrTasks + 1;
    kp=[];
    for i=1:nWindFarms
        kp(i)=0;
        for n=1:nPrevTasksFarm(i)
            TTask(p) = TPrevTask;
            Type(p) = nTypes;
            TechTask(p)= TechTaskPrev;
            ExtraOrdinaryTask(p) = 0; 
            VesselsTasks(p,:) = 1; 
            AlarmResultingInTask(p) = 0; 
            p=p+1;
            kp(i)=kp(i)+1;
        end
    end


    % Adding preventive tasks to WindFarmTasks and TasksWindFarm
        WindFarmPrevTasks = zeros(nWindFarms, max(nPrevTasksFarm));
        p = nCorrTasks + 1;
        for i = 1:nWindFarms
            for m = 1:nPrevTasksFarm(i) 
                WindFarmPrevTasks(i,m) = p; 
                TasksWindFarm(p) = i; 
                p = p+1; 
            end % for m 
        end % for i 

        % WindFarmTasks(i,m) 
        A = size(WindFarmTasks,2);                      % Dummy variable with size of NewWindFarmTasks
        B = size(WindFarmPrevTasks,2);                   % Dummy variable with size of WindFarmTasks

        for i = 1:B                                     % Adding zeros for all new tasks in WindFarmTasks
            WindFarmTasks(:,A+i) = 0;   
        end % for i 

        for i = 1:nWindFarms                            % Replacing zeros with the actual tasks 
        t = 1; 
            for m = 1:size(WindFarmTasks,2)
                if t <= B 
                    if WindFarmTasks(i,m) == 0
                        WindFarmTasks(i,m) = WindFarmPrevTasks(i,t); 
                        t = t+1; 
                    end % if 
                end % if 
            end % for m 
        end % for i 

        C = size(WindFarmTasks,2)+1;                          % Dummy variable for the new size of WindFarmTasks
        for m = 1:C-1                                         % Deleting all excess columns in WindFarmTasks (colums where all rows = 0) 
            t=C-m;
            if WindFarmTasks(:,t) == 0
                WindFarmTasks(:,t) = []; 
            end % if 
        end % for m 
  

    
% Creating new parameters based on task data and the rest of the input data

    % PickUpWindFarmTasks(i,m) 
    % PickUpWindFarmTasks a WindFarmTasks that only includes the pick-up tasks 
    for i = 1:nWindFarms
        for j = 1:length(WindFarmTasks(i,:))
            if WindFarmTasks(i,j) == 0 
                PickUpWindFarmTasks(i,j) = 0;
            else
                PickUpWindFarmTasks(i,j) = WindFarmTasks(i,j)+taskDiff; 
            end % if 
        end  % for j 
    end % for i 


    % TypeSet(i,m,k): 
    % TypeSet is a set of which task types each tasks are. If TypeSet(i,m,k) = 1
    % then task m in farm i is of task type k, 0 otherwise
    TypeSet=[];
    for i = 1:nWindFarms
        for m = 1:NumberOfTasks
            for k = 1:nTypes
                if (TasksWindFarm(m) == i) && (Type(m) == k)
                    TypeSet(i,m,k) = 1; 
                else
                    TypeSet(i,m,k) = 0; 
                end 
            end % k 
        end % m 
    end % i 

    % TTransMin(v,m):   
    % TTransMin contains the minimum transport time for vessel v to the turbine
    % corresponding to m (which is either the distance to the farm for CTVs and
    % SESes, or 0 for AVs, because they can be located in the farm already)
    TTransMin= [];
    for v = 1:nVessels
        for m=1:NumberOfTasks
            for i = 1:nWindFarms
                if TasksWindFarm(m) == i
                   TTransMin(v,m) = TTrans(1,i+1,v);
                end % if 
            end % for i 
        end % for m
    end % for v

    for v = 1:nAV
        TTransMin(v,:) = 0;
    end % for v
    
 

    % CorrPunish(m,s)
    %The penalty cost of not completing a corrective tasks
    CorrPunish = zeros(NumberOfTasks,1); 
    for m = 1:NumberOfTasks
        for i = 1:nWindFarms
                if Type(m) < nTypes
                       CorrPunish(m) =  floor((LargestTurbineEffect)*ElRevenue(Season)*nTShift*2);
                end  % if 
        end % for i

    end % for m

    % CorrPunishUnfinished(m), NoPrevPunishUnfinished(m):
    % The penalty costs of not starting a tasks that there are not enough
    % time to complete
    CorrPunishUnfinished = zeros(NumberOfTasks,1); 
    NoPrevPunishUnfinished = zeros(NumberOfTasks,1); 
    for m = 1:NumberOfTasks
        if Type(m) < nTypes
            CorrPunishUnfinished(m) = floor(ElRevenue(Season)*LargestTurbineEffect + ((2*((Distance(1,(LongestFarmDistance+1))/SpeedAV)*CostAV))+CIntTrans(1))/TMin)+2;
        end % if 
    end % for m 
    for m = 1:NumberOfTasks
        if Type(m) == nTypes
        NoPrevPunishUnfinished(m) = floor(ElRevenue(Season)*LargestTurbineEffect + ((2*((Distance(1,(LongestFarmDistance+1))/SpeedAV)*CostAV))+CIntTrans(1))/TMin)+1;
        end % if 
    end % for m 

    % TurbineSize(m)
    for m = 1:NumberOfTasks
        for i = 1:nWindFarms
            if TasksWindFarm(m) == i
                TurbineSize(m) = WindFarmTurbineSize(i); 
            end 
        end % i 
    end % m 

    % CLoss(m,s): 
    % Downtime costs
    for m = 1:NumberOfTasks
        for s = 1:nShifts
            for k = (1:length(TurbineEfficiency)-1);
            if Wind(s) == k
            CLoss(m,s) = ElRevenue(Season)*TurbineSize(m)*TurbineEfficiency(k+1);
            end % if
            end % for k
        end % for s
    end % for m

    % StartExtra(m,s): 
    % StartExtra = 1 if energy production is so low that you want to perform extra preventive tasks
    StartExtra = zeros(NumberOfTasks,nShifts);
    for m = 1:NumberOfTasks
        if Type(m) == nTypes
            for s = 1:nShifts
                if CLoss(m,s) <= alphaLowWind*TurbineSize(m)*ElRevenue(Season)
                    StartExtra(m,s) = 1;
                end % if 
            end % for s
        end % if 
    end % for m


    % AvailableHours(i,v,s):
    % Available hours for performing maintenance in shift s for task v in farm i
    for s = 1:nShifts
        for i = 1:nWindFarms
            for v = 1:nAV
                AvailableHours(i,v,s) = UWindow(v,s) - LWindow(v,s); 
            end % for v = AV
            for v = (nAV+1):nVessels
                if (UWindow(v,s) < nTShift - TTrans(1,i+1,v)) && (LWindow(v,s) < TTrans(1,i+1,v)) % vinduet pnes tidligere enn transporttid
                   if UWindow(v,s) - LWindow(v,s) < TTrans(1,i+1,v)
                       AvailableHours(i,v,s) = 0;
                   else 
                        AvailableHours(i,v,s)= UWindow(v,s) - TTrans(1,i+1,v);
                   end
                elseif (UWindow(v,s) > nTShift - TTrans(1,i+1,v)) && (LWindow(v,s) > TTrans(1,i+1,v)) % vinduet lukkes senere enn transporttid
                    if UWindow(v,s) - LWindow(v,s) < TTrans(1,i+1,v)
                       AvailableHours(i,v,s) = 0;
                    else
                        AvailableHours(i,v,s) = nTShift-TTrans(1,i+1,v)-LWindow(v,s);
                    end
                elseif (UWindow(v,s) < nTShift - TTrans(1,i+1,v)) && (LWindow(v,s) > TTrans(1,i+1,v)) % vinduet pnes senere og lukkes tidligere enn transporttid
                    AvailableHours(i,v,s) = UWindow(v,s) - LWindow(v,s);
                elseif (UWindow(v,s) > nTShift - TTrans(1,i+1,v)) && (LWindow(v,s) < TTrans(1,i+1,v)) % vinduet er pner fr og lukkes senere enn transporttid
                    AvailableHours(i,v,s) = nTShift - 2*TTrans(1,i+1,v);
                end % if 
            end % for v = SES og CTV
        end % for i
    end % for s


    PreviousTaskEnd = zeros(1,NumberOfTasks); % Vector of tasks that were completed in previous shift (which are 0 in instance 1)





%----------------------------------------------------------------------
% CREATING TEXTFILE TO RUN IN XPRESS 
%----------------------------------------------------------------------
for iteration = 1:nIterations
    
    name = [Instance,'_iteration_',num2str(iteration)];
    filename = [name,'.txt'];
    fid = fopen(filename, 'w');
    fprintf(fid, 'nWindFarms: %d \n',nWindFarms);
    fprintf(fid, 'nCorrTasks: %d \n',nCorrTasks);
    fprintf(fid, 'nPrevTasks: %d \n',nPrevTasks);
    fprintf(fid, 'nTypes: %d \n',nTypes);
    fprintf(fid, 'nAV: %d \n',nAV);
    fprintf(fid, 'nCTVuSES: %d \n',nCTV);
    fprintf(fid, 'nSES: %d \n',nSES);
    fprintf(fid, 'nShifts: %d \n',nShifts);
    fprintf(fid, 'nPrevTasksWant: %d \n',nPrevTasksWant);
    fprintf(fid, 'TDay: %d \n',TDay);
    fprintf(fid, '\n\n');

    fprintf(fid, 'CorrTasks: [');                               % Only printing tasks that are not completed in previous shifts, i.e. TTask >0
    for m = 1:length(Type)
        if TTask(m) > 0.01
            if Type(m) < nTypes
                fprintf(fid, '%d ', m);
            end % if 
        end % if 
    end % for m 
    fprintf(fid, ']\n');
    fprintf(fid, 'PrevTasks: [');
    for m = 1:length(Type)
        if TTask(m) > 0.01
            if Type(m) == nTypes
                fprintf(fid, '%d ', m);
            end % if 
        end % if 
    end % for m
    fprintf(fid, ']\n\n\n');


    fprintf(fid, 'CorrTasksPick: [');                               % Only printing tasks that are not completed in previous shifts, i.e. TTask >0
    for m = 1:length(Type)
        if TTask(m) > 0.01
            if Type(m) < nTypes
                fprintf(fid, '%d ', m+taskDiff);
            end % if 
        end % if 
    end % for m 
    fprintf(fid, ']\n');
    fprintf(fid, 'PrevTasksPick: [');
    for m = 1:length(Type)
        if TTask(m) > 0.01
            if Type(m) == nTypes
                fprintf(fid, '%d ', m+taskDiff);
            end % if 
        end % if 
    end % for m
    fprintf(fid, ']\n\n\n');

    fprintf(fid, 'TaskLast: [%d] \n', endDepot);
    fprintf(fid, 'TasksDiff: [%d] \n', taskDiff);
    fprintf(fid, 'TaskFirst: [0] \n');
    fprintf(fid, '\n\n\n');


    fprintf(fid, 'WindFarmTasks: [\n');
    fprintf(fid, '(0)    [ 0 ]\n'); 
    for i = 1:nWindFarms
      fprintf(fid, '(%d)    [ ', i);  
        for j = 1:length(WindFarmTasks(i,:))
            if WindFarmTasks(i,j) >0
                if TTask(WindFarmTasks(i,j)) > 0.01
                    fprintf(fid, '%d ', WindFarmTasks(i,j)); 
                end % if 
            end % if 
        end % for j 
        for j = 1:length(PickUpWindFarmTasks(i,:))
            if PickUpWindFarmTasks(i,j) >0
                if TTask(WindFarmTasks(i,j)) > 0.01
                    fprintf(fid, '%d ', PickUpWindFarmTasks(i,j)); 
                end % if 
            end % if 
        end % for j   
        fprintf(fid, ']\n');  
    end % for i
    fprintf(fid, '(%d)    [ %d ] \n', (nWindFarms+1),endDepot);
    fprintf(fid, ']\n\n\n');


    fprintf(fid, 'VesselsTasks: [\n');
    for v = 1:nVessels
        fprintf(fid, '(%d %d) %d ',0,v, 1);
    end % for v
    fprintf(fid, '\n');
    for m=1:size(VesselsTasks,1)
        if TTask(m) > 0.01 
            for v = 1:nAV
                fprintf(fid, '(%d %d) %d ',m,v, VesselsTasks(m,1));
            end
            for v = (nAV+1):(nAV+nSES)
                fprintf(fid, '(%d %d) %d ',m,v, VesselsTasks(m,2));
            end 
            for v = (nAV+nSES+1):(nAV+nSES+nCTV)
                     fprintf(fid, '(%d %d) %d ',m,v, VesselsTasks(m,3));
            end
            fprintf(fid, '\n');
        end % if 
    end % for m 
    for m=1:size(VesselsTasks,1)
        if TTask(m) > 0.01 
            for v = 1:nAV
                fprintf(fid, '(%d %d) %d ',m+taskDiff,v, VesselsTasks(m,1));
            end
            for v = (nAV+1):(nAV+nSES)
                fprintf(fid, '(%d %d) %d ',m+taskDiff,v, VesselsTasks(m,2));
            end 
            for v = (nAV+nSES+1):(nAV+nSES+nCTV)
                     fprintf(fid, '(%d %d) %d ',m+taskDiff,v, VesselsTasks(m,3));
            end
            fprintf(fid, '\n');
        end % if 
    end % for m
    for v = 1:nVessels
        fprintf(fid, '(%d %d) %d ',endDepot,v, 1);
    end % for v
    fprintf(fid, '\n');
    fprintf(fid, ']\n\n\n');

    fprintf(fid, 'ExtraOrdinaryTask: [\n');
    for m = 1:length(ExtraOrdinaryTask)
        if TTask(m) > 0.01
            fprintf(fid, '(%d) %d ', m, ExtraOrdinaryTask(m));
            fprintf(fid, '\n');
        end % if 
    end % for m
    fprintf(fid, ']\n\n\n');


    fprintf(fid, 'TypeSet: [\n');
    for i = 1:nWindFarms
        for m = 1:NumberOfTasks
            if TTask(m) > 0.01 
                for k = 1:length(TypeTimes)
                   fprintf(fid, '(%d %d %d) %d ', i, m, k, TypeSet(i,m,k));
                end % for k
            end % if 
        end % for m 
        fprintf(fid, '\n');
    end % for i 
    fprintf(fid, ']\n\n\n');



    fprintf(fid, 'CTrans: [\n');
    for i = 1:(nWindFarms+2)
        for j = 1:(nWindFarms+2)
            for v = 1:nVessels
                fprintf(fid, '(%d %d %d) %.2f ', (i-1), (j-1), v, CTrans(i,j,v));
            end % for v
        end % for j 
        fprintf(fid, '\n');
    end % for i 
    fprintf(fid, ']\n\n\n');


    fprintf(fid, 'TTrans: [\n');
    for i = 1:(nWindFarms+2)
        for j = 1:(nWindFarms+2)
            for v = 1:nVessels
                fprintf(fid, '(%d %d %d) %.2f ', (i-1), (j-1), v, TTrans(i,j,v));
            end % for v
        end % for j 
        fprintf(fid, '\n');
    end % for i 
    fprintf(fid, ']\n\n\n');


    fprintf(fid, 'TTask: [\n');
    fprintf(fid, '(%d) %.2f\n',0, 0); 
    for m = 1: length(TTask)
        if TTask(m) > 0.01
            fprintf(fid, '(%d) %.2f\n', m, TTask(m)); 
        end % i f
    end % for i 
    for m = 1: length(TTask)
        if TTask(m) > 0.01
            fprintf(fid, '(%d) %.2f\n',taskDiff+m, 0); 
        end  %if
    end % for i 
    fprintf(fid, '(%d) %.2f\n',endDepot, 0); 
    fprintf(fid, ']\n\n\n');


    fprintf(fid, 'CLoss: [\n');
    for m=1:size(CLoss,1)
        if TTask(m) > 0.01
            for s = 1:size(CLoss,2)
                fprintf(fid, '(%d %d) %.2f  ' ,m, s, CLoss(m,s));
            end % for s
            fprintf(fid, '\n');
        end % if 
    end % for m
    fprintf(fid, ']\n\n\n');


    fprintf(fid, 'DStart: [\n');
    for i = 1: length(DStart)
       fprintf(fid, '%d\n', DStart(i)); 
    end % for i 
    fprintf(fid, ']\n\n\n');


    fprintf(fid, 'DLimit: [\n');
    if nAV >=1
       fprintf(fid, '%d\n', DLimit); 
    end
    fprintf(fid, ']\n\n\n');

    fprintf(fid, 'PStart: [\n');
    for i=1:size(PStart,1)
        fprintf(fid, '%d ', PStart(i,:));
        fprintf(fid, '\n');
    end % for i 
    fprintf(fid, ']\n\n\n');

    fprintf(fid, 'TMin: [\n');
    for i = 1: length(TMin)
       fprintf(fid, '%d\n', TMin(i)); 
    end % for i 
    fprintf(fid, ']\n\n\n');

    fprintf(fid, 'TTransMin: [\n');
    for v = 1:nVessels
        fprintf(fid, '(%d %d) %.2f  ',v,0, 0);
        for m = 1:size(TTransMin,2)
            if TTask(m) > 0.01
                fprintf(fid, '(%d %d) %.2f  ',v,m, TTransMin(v,m));
            end % if 
        end % m 
        fprintf(fid, '(%d %d) %.2f  ',v,endDepot, 0);
        fprintf(fid, '\n');
    end 
    fprintf(fid, ']\n\n\n');

    fprintf(fid, 'TShift: [\n');
    for i = 1:forecastingPeriod
       fprintf(fid, '%d\n', TShift(i)); 
    end % for i 
    fprintf(fid, ']\n\n\n');

    fprintf(fid, 'NightCost: [\n');
    for i = 1: length(NightCostOut)
       fprintf(fid, '%d\n', NightCostOut(i)); 
    end % for i 
    fprintf(fid, ']\n\n\n');


    fprintf(fid, 'Wind: [\n');
    for s = 1:nShifts
        fprintf(fid, '%d ', Wind(s));
    end % for s
    fprintf(fid, ']\n\n\n');


    fprintf(fid, 'LWindow: [\n');
    for v =1:size(LWindow,1)
            fprintf(fid, '%d ', LWindow(v,:));
            fprintf(fid, '\n');
    end %v 
    fprintf(fid, ']\n\n\n');


    fprintf(fid, 'UWindow: [\n');
    for v=1:size(UWindow,1)
        fprintf(fid, '%d ', UWindow(v,:));
        fprintf(fid, '\n');
    end % for v 
    fprintf(fid, ']\n\n\n');


    fprintf(fid, 'AvailableHours: [\n');
    for i = 1:(nWindFarms)
        for s = 1:nShifts
            for v = 1:nVessels
                fprintf(fid, '(%d %d %d) %.2f ', i, v, s, AvailableHours(i,v,s));
            end % for v
        end % for s 
        fprintf(fid, '\n');
    end % for i 
    fprintf(fid, ']\n\n\n');


fprintf(fid, 'NoPrevPunish: [\n');                            
for m = 1:length(CorrPunish)
    if TTask(m) > 0.01 && Type(m) == nTypes
        fprintf(fid,'(%d) %d ',m, NoPrevPunish);
        fprintf(fid, '\n');
    end 
end % m
fprintf(fid, ']\n\n\n');


    fprintf(fid, 'CorrPunish: [\n');                            
    for m = 1:length(CorrPunish)
        if TTask(m) > 0.01 && Type(m) < nTypes
            fprintf(fid,'(%d) %d ',m, CorrPunish(m));
            fprintf(fid, '\n');
        end 
    end % m
    fprintf(fid, ']\n\n\n');

    fprintf(fid, 'TechTransfer: ');
    fprintf(fid, '%.2f', TechTransfer); 
    fprintf(fid, '\n\n\n');

    fprintf(fid, 'IntTransport: [\n');
    for v=1:size(IntTransport,1)
            fprintf(fid, '%.2f ', IntTransport(v,:));
            fprintf(fid, '\n');
    end % for v
    fprintf(fid, ']\n\n\n');


    fprintf(fid, 'TechCap: [\n');
    for v =1:length(TechCap)
            fprintf(fid, '%d ', TechCap(v));
            fprintf(fid, '\n');
    end %v 
    fprintf(fid, ']\n\n\n');

    fprintf(fid, 'TechTask: [\n');
    fprintf(fid, '(%d) %d\n',0, 0);
    for m = 1: length(TechTask)
        if TTask(m) > 0.01 
            fprintf(fid, '(%d) %d\n',m, TechTask(m)); 
        end % if 
    end % for i 
    for m = 1: length(TechTask)
            if TTask(m) > 0.01 
                fprintf(fid, '(%d) %d\n', m+taskDiff, (-1)*TechTask(m)); 
            end % if 
    end % for i 
    fprintf(fid, '(%d) %d\n', endDepot, 0); 
    fprintf(fid, ']\n\n\n');

    fprintf(fid, 'CorrPunishUnfinished: [\n');                  
    for m = 1:length(CorrPunishUnfinished)
        if TTask(m) > 0.01 
            if Type(m) < nTypes
                fprintf(fid,'(%d) %d ',m, CorrPunishUnfinished(m));
                fprintf(fid, '\n');
            end % if 
        end 
    end % m
    fprintf(fid, ']\n\n\n');

    fprintf(fid, 'NoPrevPunishUnfinished: [\n');                
    for m = 1:length(NoPrevPunishUnfinished)
        if TTask(m) > 0.01
            if Type(m) == nTypes
                fprintf(fid,'(%d) %d ',m, NoPrevPunishUnfinished(m));
                fprintf(fid, '\n');
            end % if 
        end 
    end % m
    fprintf(fid, ']\n\n\n');

    fprintf(fid, 'StartExtra: [\n');                            
    for m = 1:size(StartExtra,1)
        if TTask(m) > 0.01 && Type(m) == nTypes
        for s = 1:nShifts
        fprintf(fid,'(%d %d) %d  ', m, s, StartExtra(m,s));
        end
        fprintf(fid, '\n');
        end 
    end % m
    fprintf(fid, ']\n\n\n');


    fprintf(fid, 'CIntTrans: [\n');
    if nAV > 0
        for v = 1:nAV
            fprintf(fid, '%.2f ', CIntTrans(1));
            fprintf(fid, '\n');
        end
    end
    if nSES > 0
        for v = 1:nSES
            fprintf(fid, '%.2f ', CIntTrans(2));
            fprintf(fid, '\n');
        end
    end 
    if nCTV > 0
        for v = 1:nCTV
            fprintf(fid, '%.2f ', CIntTrans(3));
            fprintf(fid, '\n');
        end
    end 
    fprintf(fid, ']\n\n\n');


    fprintf(fid, 'AlarmResultingInTask: [ \n');
    for m = 1:length(AlarmResultingInTask)
        if TTask(m) > 0.01
            fprintf(fid,'(%d) %d  \n',m, AlarmResultingInTask(m));
        end % if 
    end % for m
    fprintf(fid, '\n');
    fprintf(fid, ']\n\n\n');
% -------------------------------------------

    if fid == -1
        disp('Error opening file');
    else 
        disp('Opening file = success');
    end
    fclose(fid);

if MoselRunning 
%----------------------------------------------------------------------
% RUNNING FILE IN XPRESS 
%----------------------------------------------------------------------

    % Name of datafile (instance file): 
    DataFile = filename;

    % Run model in Xpress
    [retcode, exitcode] = moselexec(modelname);

%----------------------------------------------------------------------
% RESULTS FROM XPRESS
%----------------------------------------------------------------------
end % MoselRunning 



if MoselRunning 
% Loading from Xpress: 
    % f_ms (TaskEnd) to check if task m is finished shift 1
    TaskEndThisShift;
    % w_ivs (EndNode) to check where the AV is located after shift 1 
   if nAV > 0
    EndNode;  
   end  
    % tau_mvs (StartTask) to check how much is done of tasks that are not done
    StartTaskDel;        % Cannot get deliverytasks and pick-up tasks in the same variable without editing structure of Xpress code
    StartTaskPU;         % Therefore divides StartTask into two variables, where StartTaskPU needs some editing (removing rows before Pick-up tasks begin)
    
    % Removing rows reprenting delivery tasks from StartTaskPU:
    StartTaskPUDummy = zeros(NumberOfTasks,nVessels);
    for m = 1:NumberOfTasks
        for v = 1:nVessels
           StartTaskPUDummy(m,v) = StartTaskPU(m+taskDiff,v); 
        end 
    end 
    StartTaskPUDummy;
    StartTaskPU = []; 
    StartTaskPU = StartTaskPUDummy;


% Processing data from Xpress to fit matlab 

    % Summing StartTaskDel and StartTaskPU for each task: 
    SumStartTaskDel = sum(StartTaskDel,2);
    SumStartTaskPU = sum(StartTaskPU,2);

    % Fixing TaskEnd:  TaskEnd from Xpress only includes the tasks of instance i, not tasks completed in previous shifts. TaskEnd is therefore updated task end to include all tasks, not only the tasks of the previous instance  
    if iteration == 1
        TaskEnd = TaskEndThisShift;
    else 
        for m = 1:length(TaskEnd)            
            if TaskEnd(m) == 0
                TaskEnd(m) = TaskEndThisShift(m);
            end 
        end % for m
        for m = (length(TaskEnd)+1):length(TaskEndThisShift)        
            TaskEnd(m) = TaskEndThisShift(m);
        end 
    end % if 
 
end % if MoselRunning 


%----------------------------------------------------------------------
% UPDATING INSTANCE -> SHIFT = SHIFT + 1 
%----------------------------------------------------------------------

% SHIFT:
    % Removing shift that is over: 
        Wind(1) = [];
        LWindow(:,1) = [];
        UWindow(:,1) = [];

    % Adding new shifts
        Wind(forecastingPeriod) = WindLongHorizon(forecastingPeriod+iteration);
        LWindow(:,forecastingPeriod) = StartWindow(:,(forecastingPeriod+iteration));
        UWindow(:,forecastingPeriod) = EndWindow(:,(forecastingPeriod+iteration));

     if nAV > 0   
% LOCATION OF AVs:
    % PStart(i,v):  
    AVinDepot = zeros(1,nAV); 
    for v = 1:nAV
        AVinDepot(v) = sum(EndNode,1);              % If AV ends in depot, AVinDepot = 0
    end % for v


    PStart = zeros(nWindFarms+2,nAV); 
    for v = 1:nAV
        for i = 1:nWindFarms
            PStart(i+1,v) = EndNode(i,v); 
            if AVinDepot(v) == 0
                PStart(nWindFarms+2) = 1; 
            elseif AVinDepot(v) == 1
                PStart(nWindFarms+2) = 0;
            end % if
        end % i 
    end % v
   

    % Making PStart integer: 
    for i = 1:size(PStart,1)
        for j = 1:size(PStart,2)
            if PStart(i,j) <= 0.5
                PStart(i,j) = 0;
            elseif PStart(i,j) > 0.5
                PStart(i,j) = 1;
            end % if 
        end % for j
    end % for i 


    % DStart(v) 
    for v = 1:nAV
        if PStart(nWindFarms+2) == 1        % If AV starts in depot, DStart = 0
            DStart(v) = 0; 
        else
            DStart(v) = DStart(v) + 1; 
        end % if 
    end % v
     end %  if nAV > 0   
 
% TASKS:
    % Updating for current tasks: 
    % How much is performed of current tasks: 
        TimeWorkedOnTasks = [];
        for m = 1:NumberOfTasks
            if SumStartTaskPU(m) == 0; 
                TimeWorkedOnTasks(m) = 0; 
            else
                TimeWorkedOnTasks(m) = SumStartTaskPU(m)-(SumStartTaskDel(m)+TechTransfer);
            end % if 
        end % for m

    % How much is left on current tasks: 
        for m = 1:NumberOfTasks 
            TTask(m) = TTask(m) - TimeWorkedOnTasks(m);       
            if TTask(m) < 0
                TTask(m) = 0; 
            end % if 
        end % for m
 
    % The number of current tasks that are not completed: 
        TasksNotCompleted = 0; 
        for m = 1:length(TTask)
            if TTask(m) > 0.01
            TasksNotCompleted = TasksNotCompleted + 1; 
            end % if
        end % for m

 % New tasks due to alarms checked the previous shift: 
    NewnCorrTasks = 0; 
    NewTasksWindFarm = zeros(nWindFarms,1);
    NewTasks = zeros(nWindFarms,1); 
    counter = NumberOfTasks; 
    NewWindFarmsSize = zeros(1,nWindFarms); 
    counter2 = 0; 
    for m = 1:NumberOfTasks
        if AlarmResultingInTask(m) == 1
            counter2 = counter2 + 1;
            if TaskEndThisShift(m) == 1 
                counter = counter +1; 
                TTask(counter) = AlarmTTask(counter2); 
                TechTask(counter) = AlarmTechTask(counter2); 
                Type(counter) = AlarmType(counter2); 
                AlarmResultingInTask(counter) = 0; 
                ExtraOrdinaryTask(counter) = AlarmExtraOrdinaryTask(counter2);  
                TasksWindFarm(counter) = AlarmTasksWindFarm(counter2);
                VesselsTasks(counter,:) = AlarmVesselsTasks(counter2,:);
                NewTasks(counter-NumberOfTasks) = counter;
                NewTasksWindFarm(counter-NumberOfTasks) = AlarmTasksWindFarm(counter2);
                for i = 1:nWindFarms
                    if AlarmTasksWindFarm(counter2) == i
                        NewWindFarmsSize(i) = NewWindFarmsSize(i) + 1; 
                    end % i 
                end %for i 
            end % if
        end % if 
    end % for m 

    % WindFarmsTasks 
    if sum(NewWindFarmsSize) == 0
            NewWindFarmTasks = zeros(nWindFarms,1); 
    else 
        NewWindFarmTasks = zeros(nWindFarms,max(NewWindFarmsSize)); 
    end 
    for i = 1:nWindFarms
        t = 1; 
        for m = 1:length(NewTasksWindFarm)
            if NewTasksWindFarm(m) == i
                NewWindFarmTasks(i,t) = NewTasks(m);
                t = t + 1; 
            end % if 
        end % for m 
    end % for i 
    
    % WindFarmTasks: 
    A = size(WindFarmTasks,2);                      % Dummy variable with size of NewWindFarmTasks
    B = size(NewWindFarmTasks,2);                   % Dummy variable with size of WindFarmTasks
    Diff = 0;
    if NewWindFarmTasks(1,1) >0
        Diff = NumberOfTasks - NewWindFarmTasks(1,1); 
    elseif NewWindFarmTasks(1,1) == 0
        Diff = NumberOfTasks - NewWindFarmTasks(2,1);
    end 
    
    % Updating task numbers in NewWindFarmTasks
    for i = 1:size(NewWindFarmTasks,1)
        for m = 1:size(NewWindFarmTasks,2)
            if NewWindFarmTasks(i,m) > 0 
                NewWindFarmTasks(i,m) = NewWindFarmTasks(i,m)+1+Diff; 
            end % if
        end % for m 
    end % for i 

    for i = 1:B                                     % Adding zeros for all new tasks in WindFarmTasks
        WindFarmTasks(:,A+i) = 0;   
    end % for i 

    for i = 1:nWindFarms                            % Replacing zeros with the actual tasks 
    t = 1; 
        for m = 1:size(WindFarmTasks,2)
            if t <= B 
                if WindFarmTasks(i,m) == 0
                    WindFarmTasks(i,m) = NewWindFarmTasks(i,t); 
                    t = t+1; 
                end % if 
            end % if 
        end % for m 
    end % for i 

    C = size(WindFarmTasks,2)+1;                          % Dummy variable for the new size of WindFarmTasks
    for m = 1:C-1                                         % Deleting all excess columns in WindFarmTasks (colums where all rows = 0) 
        t=C-m;
        if WindFarmTasks(:,t) == 0
            WindFarmTasks(:,t) = []; 
        end % if 
    end % for m 
    

    NumberOfTasks = counter;


% New corrective tasks: 

    NewWindFarmTasks = eval(['WindFarmTasks_' num2str(iteration+1)]);          % Collecting new tasks from shift "period"
    NewVesselsTasks = eval(['VesselsTasks_' num2str(iteration+1)]);
    NewType = eval(['Type_' num2str(iteration+1)]);
    NewTTask = eval(['TTask_' num2str(iteration+1)]);
    NewTechTask = eval(['TechTask_' num2str(iteration+1)]);
    NewnCorrTasks = eval(['nCorrTasks_' num2str(iteration+1)]);
    NewTasksWindFarm = eval(['TasksWindFarm_' num2str(iteration+1)]);
    NewAlarmResultingInTask = eval(['AlarmResultingInTask_' num2str(iteration+1)]);
    NewExtraOrdinaryTask = eval(['ExtraOrdinaryTask_' num2str(iteration+1)]);
    
    for m = 1:(NewnCorrTasks)
        TTask(m+NumberOfTasks) = NewTTask(m);
        VesselsTasks(m+NumberOfTasks,:) = NewVesselsTasks(m,:); 
        TechTask(m+NumberOfTasks) = NewTechTask(m);
        ExtraOrdinaryTask(m+NumberOfTasks) = NewExtraOrdinaryTask(m);
        Type(m+NumberOfTasks) = NewType(m);
        TasksWindFarm(m+NumberOfTasks) = NewTasksWindFarm(m);
        AlarmResultingInTask(m+NumberOfTasks) = NewAlarmResultingInTask(m);
    end % for m 
    
    
    % WindFarmTasks(i,m) 
    A = size(WindFarmTasks,2);                      % Dummy variable with size of NewWindFarmTasks
    B = size(NewWindFarmTasks,2);                   % Dummy variable with size of WindFarmTasks
    Diff = 0;
    if NewWindFarmTasks(1,1) >0
        Diff = NumberOfTasks - NewWindFarmTasks(1,1); 
    elseif NewWindFarmTasks(1,1) == 0
        Diff = NumberOfTasks - NewWindFarmTasks(2,1);
    end 
    
    % Updating task numbers in NewWindFarmTasks
    for i = 1:size(NewWindFarmTasks,1)
        for m = 1:size(NewWindFarmTasks,2)
            if NewWindFarmTasks(i,m) > 0 
                NewWindFarmTasks(i,m) = NewWindFarmTasks(i,m)+1+Diff; 
            end % if
        end % for m 
    end % for i 

    for i = 1:B                                     % Adding zeros for all new tasks in WindFarmTasks
        WindFarmTasks(:,A+i) = 0;   
    end % for i 

    for i = 1:nWindFarms                            % Replacing zeros with the actual tasks 
    t = 1; 
        for m = 1:size(WindFarmTasks,2)
            if t <= B 
                if WindFarmTasks(i,m) == 0
                    WindFarmTasks(i,m) = NewWindFarmTasks(i,t); 
                    t = t+1; 
                end % if 
            end % if 
        end % for m 
    end % for i 

    C = size(WindFarmTasks,2)+1;                          % Dummy variable for the new size of WindFarmTasks
    for m = 1:C-1                                         % Deleting all excess columns in WindFarmTasks (colums where all rows = 0) 
        t=C-m;
        if WindFarmTasks(:,t) == 0
            WindFarmTasks(:,t) = []; 
        end % if 
    end % for m 
    
    
    NumberOfTasks = NumberOfTasks + NewnCorrTasks;   


 % new preventive tasks 
      TasksFinishedLastShift = zeros(1,length(TaskEnd)); 
      for m = 1:length(TaskEnd)
            TasksFinishedLastShift(m) = TaskEnd(m) - PreviousTaskEnd(m);
      end % for m 

      nPrevTasksFarm = zeros(1,2); 
      for i = 1:nWindFarms
        for m = 1:length(TasksFinishedLastShift)
              if (Type(m) == nTypes) && (TasksWindFarm(m) == i) && (TasksFinishedLastShift(m) == 1) 
                  nPrevTasksFarm(i) = nPrevTasksFarm(i) + 1; 
              end % if 
          end % for m 
      end % for i 

    for m = 1:sum(nPrevTasksFarm)
            TTask(m+NumberOfTasks) = TPrevTask;
            VesselsTasks(m+NumberOfTasks,:) = 1; 
            TechTask(m+NumberOfTasks) = TechTaskPrev;
            ExtraOrdinaryTask(m+NumberOfTasks) = 0;
            Type(m+NumberOfTasks) = nTypes;
            AlarmResultingInTask(m+NumberOfTasks) = 0; 
    end % for m


    % Adding new preventive tasks to WindFarmTasks and TasksWindFarm
        WindFarmPrevTasks = zeros(nWindFarms, max(nPrevTasksFarm));
        p = NumberOfTasks + 1;
        for i = 1:nWindFarms
            for m = 1:nPrevTasksFarm(i) 
                WindFarmPrevTasks(i,m) = p; 
                TasksWindFarm(p) = i; 
                p = p+1; 
            end % for m 
        end % for i 
    


        % WindFarmTasks(i,m) 
        A = size(WindFarmTasks,2);                      % Dummy variable with size of NewWindFarmTasks
        B = size(WindFarmPrevTasks,2);                   % Dummy variable with size of WindFarmTasks

        for i = 1:B                                     % Adding zeros for all new tasks in WindFarmTasks
            WindFarmTasks(:,A+i) = 0;   
        end % for i 

        for i = 1:nWindFarms                            % Replacing zeros with the actual tasks 
        t = 1; 
            for m = 1:size(WindFarmTasks,2)
                if t <= B 
                    if WindFarmTasks(i,m) == 0
                        WindFarmTasks(i,m) = WindFarmPrevTasks(i,t); 
                        t = t+1; 
                    end % if 
                end % if 
            end % for m 
        end % for i 

        C = size(WindFarmTasks,2)+1;                          % Dummy variable for the new size of WindFarmTasks
        for m = 1:C-1                                         % Deleting all excess columns in WindFarmTasks (colums where all rows = 0) 
            t=C-m;
            if WindFarmTasks(:,t) == 0
                WindFarmTasks(:,t) = []; 
            end % if 
        end % for m 
 

        NumberOfTasks = NumberOfTasks + sum(nPrevTasksFarm); 

% Creating task and shift dependent parameters: 
     % PickUpWindFarmTasks 
    for i = 1:nWindFarms
        for j = 1:length(WindFarmTasks(i,:))
            if WindFarmTasks(i,j) == 0 
                PickUpWindFarmTasks(i,j) = 0;
            else
                PickUpWindFarmTasks(i,j) = WindFarmTasks(i,j)+taskDiff; 
            end % if 
        end  % for j 
    end % for i 
  

    % TypeSet(i,m,k): 
    TypeSet=[];
    for i = 1:nWindFarms
        for m = 1:NumberOfTasks
            for k = 1:nTypes
                if (TasksWindFarm(m) == i) && (Type(m) == k)
                    TypeSet(i,m,k) = 1; 
                else
                    TypeSet(i,m,k) = 0; 
                end 
            end % k 
        end % m 
    end % i 

    % TTransMin(v,m):                                        
    TTransMin= [];
    for v = 1:nVessels
        for m=1:NumberOfTasks
            for i = 1:nWindFarms
                if TasksWindFarm(m) == i
                   TTransMin(v,m) = TTrans(1,i+1,v);
                end % if 
            end % for i 
        end % for m
    end % for v

    for v = 1:nAV
        TTransMin(v,:) = 0;
    end % for v
    TTransMin;


    % CorrPunish(m) 
    CorrPunish = zeros(NumberOfTasks,1); 
    for m = 1:NumberOfTasks
        for i = 1:nWindFarms
                if Type(m) < nTypes
                       CorrPunish(m) =  floor((LargestTurbineEffect*ElRevenue(Season)*nTShift*2));
                end  % if 
        end % for i

    end % for m

    % CorrPunishUnfinished(m), NoPrevPunishUnfinished(m), 
    CorrPunishUnfinished = [];
    NoPrevPunishUnfinished =[];

    for m = 1:NumberOfTasks
        if Type(m) < nTypes
            CorrPunishUnfinished(m) = floor(ElRevenue(Season)*LargestTurbineEffect + ((2*((Distance(1,(LongestFarmDistance+1))/SpeedAV)*CostAV))+CIntTrans(1))/TMin)+2;
            NoPrevPunishUnfinished(m) = 0; 
        elseif Type(m) == nTypes
            CorrPunishUnfinished(m) = 0;
            NoPrevPunishUnfinished(m) = floor(ElRevenue(Season)*LargestTurbineEffect + ((2*((Distance(1,(LongestFarmDistance+1))/SpeedAV)*CostAV))+CIntTrans(1))/TMin)+1;
        end % if
    end % for m 
    
     % TurbineSize(m)
    for m = 1:NumberOfTasks
        for i = 1:nWindFarms
            if TasksWindFarm(m) == i
                TurbineSize(m) = WindFarmTurbineSize(i); 
            end 
        end % i 
     end % m 

    % CLoss(m,s): 
    for m = 1:NumberOfTasks
        for s = 1:nShifts
            for k = (1:length(TurbineEfficiency)-1);
            if Wind(s) == k
            CLoss(m,s) = ElRevenue(Season)*TurbineSize(m)*TurbineEfficiency(k+1);
            end % if
            end % for k
        end % for s
    end % for m

    % StartExtra(m,s): 
    StartExtra = zeros(NumberOfTasks,nShifts);
    for m = 1:NumberOfTasks
        if TTask(m) > 0.01
            if Type(m) == nTypes
                for s = 1:nShifts
                    if CLoss(m,s) <= alphaLowWind*TurbineSize(m)*ElRevenue(Season)
                        StartExtra(m,s) = 1;
                    end % if 
                end % for s
            end 
        end 
    end % for m

    % AvailableHours(i,v,s): 
    for s = 1:nShifts
        for i = 1:nWindFarms
            for v = 1:nAV
                AvailableHours(i,v,s) = UWindow(v,s) - LWindow(v,s); 
            end % for v = AV
            for v = (nAV+1):nVessels
                if (UWindow(v,s) < nTShift - TTrans(1,i+1,v)) && (LWindow(v,s) < TTrans(1,i+1,v)) % vinduet pnes tidligere enn transporttid
                   if UWindow(v,s) - LWindow(v,s) < TTrans(1,i+1,v)
                       AvailableHours(i,v,s) = 0;
                   else 
                        AvailableHours(i,v,s)= UWindow(v,s) - TTrans(1,i+1,v);
                   end
                elseif (UWindow(v,s) > nTShift - TTrans(1,i+1,v)) && (LWindow(v,s) > TTrans(1,i+1,v)) % vinduet lukkes senere enn transporttid
                    if UWindow(v,s) - LWindow(v,s) < TTrans(1,i+1,v)
                       AvailableHours(i,v,s) = 0;
                    else
                        AvailableHours(i,v,s) = nTShift-TTrans(1,i+1,v)-LWindow(v,s);
                    end
                elseif (UWindow(v,s) < nTShift - TTrans(1,i+1,v)) && (LWindow(v,s) > TTrans(1,i+1,v)) % vinduet pnes senere og lukkes tidligere enn transporttid
                    AvailableHours(i,v,s) = UWindow(v,s) - LWindow(v,s);
                elseif (UWindow(v,s) > nTShift - TTrans(1,i+1,v)) && (LWindow(v,s) < TTrans(1,i+1,v)) % vinduet er pner fr og lukkes senere enn transporttid
                    AvailableHours(i,v,s) = nTShift - 2*TTrans(1,i+1,v);
                end % if 
            end % for v = SES og CTV
        end % for i
    end % for s


    % nPrevTasks(m) and nCorrTasks(m):
        nPrevTasks = 0; 
        nCorrTasks = 0; 
        for m = 1:length(TTask) 
                if TTask(m) > 0.01
                    if Type(m) == nTypes
                        nPrevTasks = nPrevTasks + 1; 
                    else
                        nCorrTasks = nCorrTasks + 1; 
                    end % if
                end % if 
        end % for m    


    % Checking how many tasks are completed by this shift 
    PreviousTaskEnd = zeros(1,NumberOfTasks);
    for m = 1:length(TaskEnd)
        PreviousTaskEnd(m) = TaskEnd(m);
    end % for m 

    
iteration = iteration +1; 
end % for iter = 1:nIterations    
    

% -------------------------------------------
% UTSKRIFTEN UNDER HER SKAL SLETTES 
% -------------------------------------------
 
if RunningOnce

name = [Instance, '_updated'];
filename = [name,'.txt'];
fid = fopen(filename, 'w');
fprintf(fid, 'nWindFarms: %d \n',nWindFarms);
fprintf(fid, 'nCorrTasks: %d \n',nCorrTasks);
fprintf(fid, 'nPrevTasks: %d \n',nPrevTasks);
fprintf(fid, 'nTypes: %d \n',nTypes);
fprintf(fid, 'nAV: %d \n',nAV);
fprintf(fid, 'nCTVuSES: %d \n',nCTV);
fprintf(fid, 'nSES: %d \n',nSES);
fprintf(fid, 'nShifts: %d \n',nShifts);
fprintf(fid, 'nPrevTasksWant: %d \n',nPrevTasksWant);
fprintf(fid, 'TDay: %d \n',TDay);
fprintf(fid, '\n\n');

fprintf(fid, 'CorrTasks: [');                               % Only printing tasks that are not done, i.e. TTask >0
for m = 1:length(Type)
    if TTask(m) > 0.01
        if Type(m) < nTypes
            fprintf(fid, '%d ', m);
        end % if 
    end % if 
end % for m 
fprintf(fid, ']\n');
fprintf(fid, 'PrevTasks: [');
for m = 1:length(Type)
    if TTask(m) > 0.01
        if Type(m) == nTypes
            fprintf(fid, '%d ', m);
        end % if 
    end % if 
end % for m
fprintf(fid, ']\n\n\n');


fprintf(fid, 'CorrTasksPick: [');                               % Only printing tasks that are not done, i.e. TTask >0
for m = 1:length(Type)
    if TTask(m) > 0.01
        if Type(m) < nTypes
            fprintf(fid, '%d ', m+taskDiff);
        end % if 
    end % if 
end % for m 
fprintf(fid, ']\n');
fprintf(fid, 'PrevTasksPick: [');
for m = 1:length(Type)
    if TTask(m) > 0.01
        if Type(m) == nTypes
            fprintf(fid, '%d ', m+taskDiff);
        end % if 
    end % if 
end % for m
fprintf(fid, ']\n\n\n');

fprintf(fid, 'TaskLast: [%d] \n', endDepot);
fprintf(fid, 'TasksDiff: [%d] \n', taskDiff);
fprintf(fid, 'TaskFirst: [0] \n');
fprintf(fid, '\n\n\n');


fprintf(fid, 'WindFarmTasks: [\n');
fprintf(fid, '(0)    [ 0 ]\n'); 
for i = 1:nWindFarms
  fprintf(fid, '(%d)    [ ', i);  
    for j = 1:length(WindFarmTasks(i,:))
        if WindFarmTasks(i,j) > 0
            if TTask(WindFarmTasks(i,j)) > 0.01
                fprintf(fid, '%d ', WindFarmTasks(i,j)); 
            end % if 
        end % if 
    end % for j 
    for j = 1:length(PickUpWindFarmTasks(i,:))
        if PickUpWindFarmTasks(i,j) >0
            if TTask(WindFarmTasks(i,j)) > 0.01
                fprintf(fid, '%d ', PickUpWindFarmTasks(i,j)); 
            end % if 
        end % if 
    end % for j   
    fprintf(fid, ']\n');  
end % for i
fprintf(fid, '(%d)    [ %d ] \n', (nWindFarms+1),endDepot);
fprintf(fid, ']\n\n\n');


fprintf(fid, 'VesselsTasks: [\n');
for v = 1:nVessels
    fprintf(fid, '(%d %d) %d ',0,v, 1);
end % for v
fprintf(fid, '\n');
for m=1:size(VesselsTasks,1)
    if TTask(m) > 0.01 
        for v = 1:nAV
            fprintf(fid, '(%d %d) %d ',m,v, VesselsTasks(m,1));
        end
        for v = (nAV+1):(nAV+nSES)
            fprintf(fid, '(%d %d) %d ',m,v, VesselsTasks(m,2));
        end 
        for v = (nAV+nSES+1):(nAV+nSES+nCTV)
                 fprintf(fid, '(%d %d) %d ',m,v, VesselsTasks(m,3));
        end
        fprintf(fid, '\n');
    end % if 
end % for m 
for m=1:size(VesselsTasks,1)
    if TTask(m) > 0.01 
        for v = 1:nAV
            fprintf(fid, '(%d %d) %d ',m+taskDiff,v, VesselsTasks(m,1));
        end
        for v = (nAV+1):(nAV+nSES)
            fprintf(fid, '(%d %d) %d ',m+taskDiff,v, VesselsTasks(m,2));
        end 
        for v = (nAV+nSES+1):(nAV+nSES+nCTV)
                 fprintf(fid, '(%d %d) %d ',m+taskDiff,v, VesselsTasks(m,3));
        end
        fprintf(fid, '\n');
    end % if 
end % for m
for v = 1:nVessels
    fprintf(fid, '(%d %d) %d ',endDepot,v, 1);
end % for v
fprintf(fid, '\n');
fprintf(fid, ']\n\n\n');

fprintf(fid, 'ExtraOrdinaryTask: [\n');
for m = 1:length(ExtraOrdinaryTask)
    if TTask(m) > 0.01
        fprintf(fid, '(%d) %d ', m, ExtraOrdinaryTask(m));
        fprintf(fid, '\n');
    end % if 
end % for m
fprintf(fid, ']\n\n\n');


fprintf(fid, 'TypeSet: [\n');
for i = 1:nWindFarms
    for m = 1:NumberOfTasks
        if TTask(m) > 0.01 
            for k = 1:length(TypeTimes)
               fprintf(fid, '(%d %d %d) %d ', i, m, k, TypeSet(i,m,k));
            end % for k
        end % if 
    end % for m 
    fprintf(fid, '\n');
end % for i 
fprintf(fid, ']\n\n\n');



fprintf(fid, 'CTrans: [\n');
for i = 1:(nWindFarms+2)
    for j = 1:(nWindFarms+2)
        for v = 1:nVessels
            fprintf(fid, '(%d %d %d) %.2f ', (i-1), (j-1), v, CTrans(i,j,v));
        end % for v
    end % for j 
    fprintf(fid, '\n');
end % for i 
fprintf(fid, ']\n\n\n');


fprintf(fid, 'TTrans: [\n');
for i = 1:(nWindFarms+2)
    for j = 1:(nWindFarms+2)
        for v = 1:nVessels
            fprintf(fid, '(%d %d %d) %.2f ', (i-1), (j-1), v, TTrans(i,j,v));
        end % for v
    end % for j 
    fprintf(fid, '\n');
end % for i 
fprintf(fid, ']\n\n\n');


fprintf(fid, 'TTask: [\n');
fprintf(fid, '(%d) %.2f\n',0, 0); 
for m = 1: length(TTask)
    if TTask(m) > 0.01
        fprintf(fid, '(%d) %.2f\n', m, TTask(m)); 
    end % i f
end % for i 
for m = 1: length(TTask)
    if TTask(m) > 0.01
        fprintf(fid, '(%d) %.2f\n',taskDiff+m, 0); 
    end  %if
end % for i 
fprintf(fid, '(%d) %.2f\n',endDepot, 0); 
fprintf(fid, ']\n\n\n');


fprintf(fid, 'CLoss: [\n');
for m=1:size(CLoss,1)
    if TTask(m) > 0.01
        for s = 1:size(CLoss,2)
            fprintf(fid, '(%d %d) %.2f  ' ,m, s, CLoss(m,s));
        end % for s
        fprintf(fid, '\n');
    end % if 
end % for m
fprintf(fid, ']\n\n\n');


fprintf(fid, 'DStart: [\n');
for i = 1: length(DStart)
   fprintf(fid, '%d\n', DStart(i)); 
end % for i 
fprintf(fid, ']\n\n\n');


fprintf(fid, 'DLimit: [\n');
if nAV >=1
   fprintf(fid, '%d\n', DLimit); 
end
fprintf(fid, ']\n\n\n');

fprintf(fid, 'PStart: [\n');
for i=1:size(PStart,1)
    fprintf(fid, '%d ', PStart(i,:));
    fprintf(fid, '\n');
end % for i 
fprintf(fid, ']\n\n\n');

fprintf(fid, 'TMin: [\n');
for i = 1: length(TMin)
   fprintf(fid, '%d\n', TMin(i)); 
end % for i 
fprintf(fid, ']\n\n\n');

fprintf(fid, 'TTransMin: [\n');
for v = 1:nVessels
    fprintf(fid, '(%d %d) %.2f  ',v,0, 0);
    for m = 1:size(TTransMin,2)
        if TTask(m) > 0.01
            fprintf(fid, '(%d %d) %.2f  ',v,m, TTransMin(v,m));
        end % if 
    end % m 
    fprintf(fid, '(%d %d) %.2f  ',v,endDepot, 0);
    fprintf(fid, '\n');
end 
fprintf(fid, ']\n\n\n');

fprintf(fid, 'TShift: [\n');
for i = 1:forecastingPeriod
   fprintf(fid, '%d\n', TShift(i)); 
end % for i 
fprintf(fid, ']\n\n\n');

fprintf(fid, 'NightCost: [\n');
for i = 1: length(NightCostOut)
   fprintf(fid, '%d\n', NightCostOut(i)); 
end % for i 
fprintf(fid, ']\n\n\n');


fprintf(fid, 'Wind: [\n');
for s = 1:nShifts
    fprintf(fid, '%d ', Wind(s));
end % for s
fprintf(fid, ']\n\n\n');


fprintf(fid, 'LWindow: [\n');
for v =1:size(LWindow,1)
        fprintf(fid, '%d ', LWindow(v,:));
        fprintf(fid, '\n');
end %v 
fprintf(fid, ']\n\n\n');


fprintf(fid, 'UWindow: [\n');
for v=1:size(UWindow,1)
    fprintf(fid, '%d ', UWindow(v,:));
    fprintf(fid, '\n');
end % for v 
fprintf(fid, ']\n\n\n');


fprintf(fid, 'AvailableHours: [\n');
for i = 1:(nWindFarms)
    for s = 1:nShifts
        for v = 1:nVessels
            fprintf(fid, '(%d %d %d) %.2f ', i, v, s, AvailableHours(i,v,s));
        end % for v
    end % for s 
    fprintf(fid, '\n');
end % for i 
fprintf(fid, ']\n\n\n');


fprintf(fid, 'NoPrevPunish: [\n');                            
for m = 1:length(CorrPunish)
    if TTask(m) > 0.01 && Type(m) == nTypes
        fprintf(fid,'(%d) %d ',m, NoPrevPunish);
        fprintf(fid, '\n');
    end 
end % m
fprintf(fid, ']\n\n\n');


fprintf(fid, 'CorrPunish: [\n');                            
for m = 1:length(CorrPunish)
    if TTask(m) > 0.01 && Type(m) < nTypes
        fprintf(fid,'(%d) %d ',m, CorrPunish(m));
        fprintf(fid, '\n');
    end 
end % m
fprintf(fid, ']\n\n\n');

fprintf(fid, 'TechTransfer: ');
fprintf(fid, '%.2f', TechTransfer); 
fprintf(fid, '\n\n\n');

fprintf(fid, 'IntTransport: [\n');
for v=1:size(IntTransport,1)
        fprintf(fid, '%.2f ', IntTransport(v,:));
        fprintf(fid, '\n');
end % for v
fprintf(fid, ']\n\n\n');


fprintf(fid, 'TechCap: [\n');
for v =1:length(TechCap)
        fprintf(fid, '%d ', TechCap(v));
        fprintf(fid, '\n');
end %v 
fprintf(fid, ']\n\n\n');

fprintf(fid, 'TechTask: [\n');
fprintf(fid, '(%d) %d\n',0, 0);
for m = 1: length(TechTask)
    if TTask(m) > 0.01 
        fprintf(fid, '(%d) %d\n',m, TechTask(m)); 
    end % if 
end % for i 
for m = 1: length(TechTask)
        if TTask(m) > 0.01 
            fprintf(fid, '(%d) %d\n', m+taskDiff, (-1)*TechTask(m)); 
        end % if 
end % for i 
fprintf(fid, '(%d) %d\n', endDepot, 0); 
fprintf(fid, ']\n\n\n');


fprintf(fid, 'CorrPunishUnfinished: [\n');                  
for m = 1:length(CorrPunishUnfinished)
    if TTask(m) > 0.01 
        if Type(m) < nTypes
            fprintf(fid,'(%d) %d ',m, CorrPunishUnfinished(m));
            fprintf(fid, '\n');
        end % if 
    end 
end % m
fprintf(fid, ']\n\n\n');

fprintf(fid, 'NoPrevPunishUnfinished: [\n');                
for m = 1:length(NoPrevPunishUnfinished)
    if TTask(m) > 0.01
        if Type(m) == nTypes
            fprintf(fid,'(%d) %d ',m, NoPrevPunishUnfinished(m));
            fprintf(fid, '\n');
        end % if 
    end 
end % m
fprintf(fid, ']\n\n\n');

fprintf(fid, 'StartExtra: [\n');                            
for m = 1:size(StartExtra,1)
    if TTask(m) > 0.01 && Type(m) == nTypes
    for s = 1:nShifts
    fprintf(fid,'(%d %d) %d  ', m, s, StartExtra(m,s));
    end
    fprintf(fid, '\n');
    end 
end % m
fprintf(fid, ']\n\n\n');


fprintf(fid, 'CIntTrans: [\n');
if nAV > 0
    for v = 1:nAV
        fprintf(fid, '%.2f ', CIntTrans(1));
        fprintf(fid, '\n');
    end
end
if nSES > 0
    for v = 1:nSES
        fprintf(fid, '%.2f ', CIntTrans(2));
        fprintf(fid, '\n');
    end
end 
if nCTV > 0
    for v = 1:nCTV
        fprintf(fid, '%.2f ', CIntTrans(3));
        fprintf(fid, '\n');
    end
end 
fprintf(fid, ']\n\n\n');

fprintf(fid, 'AlarmResultingInTask: [ \n');
for m = 1:length(AlarmResultingInTask)
    if TTask(m) > 0.01
        fprintf(fid,'(%d) %d  \n',m, AlarmResultingInTask(m));
    end % if 
end % for m
fprintf(fid, '\n');
fprintf(fid, ']\n\n\n');

% -------------------------------------------

if fid == -1
    disp('Error opening file');
else 
    disp('Opening file = success');
end
fclose(fid);

end % if RunningOnce
