clear
clc
% The scenario generator only generates corrective tasks. Preventive tasks
% are handled by the simulator 

% -----------------------------------------------------------------------------------------
% DATA INPUT
% -----------------------------------------------------------------------------------------

    % Name of the output file created
    name = '100_turbines_instans_01';                                               % Name of instance generated


    % Planning period data:
    
    Season = 1;                                                     % Season (Summer = 1, Fall = 2, Winter = 3, Spring = 4)
    numberOfSeasons = 4;                                            % 
    nShifts = 31;                                                   % Number of shifts in the simulator
    nTShift = 12;                                                   % Number of hours in each shift
    TDay = 24;                                                      % Hours in a day


    % Wind farm data:
    
    nWindFarms = 2;                                                 % Number of wind farms
    nTurbines = [100 100];                                            % Number of turbines in each wind farm 
    DistanceDepot = [70, 80];                                       % Distance from depot to each wind farm 
    DistanceFarms = [0 50; 50 0];                                   % Distance between wind farms
    WindFarmTurbineSize = [5, 3.6];                                 % Turbine efficiency in the wind farms 


    % Fleet data: 
                                                                % Fleet size:
    nAV = 1;                                                        % Number of AVs
    nCTV = 0;                                                       % Number of CTVs, SES' excluded
    nSES = 1;                                                       % Number of SES'
    nVessels = nAV + nCTV + nSES;                                   % Total number of vessels 

                                                                    % Fleet speed:
    SpeedAV = 22.2;                                                 % Speed AV in km/h
    SpeedCTV = 37.0;                                                % Speed CTV in km/h
    SpeedSES = 86.34;                                               % Speed SES in km/h

    TechTransfer = 0.5;                                             % Time to transfer technicians to/from vessel to turbine

                                                                    % Average internal travel times within a wind farm: 
    IntAV = [0.1893 0.1893];                                        % AV: wind farm 1, wind farm 2
    IntCTV = [0.1136 0.1136];                                       % CTV: wind farm 1, wind farm 2                                                             
    IntSES = [0.0487 0.0487];                                       % SES: wind farm 1, wind farm 2

                                                                    % Capacity on technicians:
    CapAV = 24;                                                     % Number of technicians AV
    CapCTV = 12;                                                    % Number of technicians CTV
    CapSES = 12;                                                    % Number of technicians SES

                                                                    % Transportation costs:
    CostAV = 1125;                                                  % Travel cost AV in EU/h
    CostCTV = 225;                                                  % Travel cost CTV in EU/h
    CostSES = 383;                                                  % Travel cost SES in EU/h

    ExtraCostOfTransfering = 0.4;                                   % 
                                                                    % Unit
                                                                    % costs per task performed by a vessel:

    StayOutCostAV = CostAV*ExtraCostOfTransfering;                                        
    StayOutCostCTV = CostCTV*ExtraCostOfTransfering; 
    StayOutCostSES = CostSES*ExtraCostOfTransfering; 

    DLimit = 28;                                                    % Maximum number of shifts an AV can stay offshore

                                                                    % Wave limits: 
    WaveLimitAV = 3.0;                                              % Wave limit AV in m
    WaveLimitCTV = 1.5;                                             % Wave limit CTV in m
    WaveLimitSES = 2.5;                                             % Wave limit SES in m

    TMin = 4;                                                       % Min weather window to leave depot = minimum task time+transfer time*2


    % Maintenance task data:
    nTypes = 6;                                                     % Total number of tasks, including both preventive types and corrective types 

                                                                    % Number of technicians needed for the different types of maintenance task types:
    TechTask1 = 2; 
    TechTask2 = 2; 
    TechTask3 = 2; 
    TechTask4 = 3; 
    TechTask5 = 4; 
    TechTaskPrev = 3; 
    TechTask = [TechTask1 TechTask2 TechTask3 TechTask4 TechTask5 TechTaskPrev];

                                                                    % Duration time maintenance tasks:
    TType1 = 0.5;                                                   % Type1 = Alarm 
    TType2=3;                                                       % Type2 = Manual reset
    TType3=7.5;                                                     % Type3 = Minor repair
    TType4=22;                                                      % Type4 = Medium repair
    TType5=26;                                                      % Type5 = Major repair
    TPrevTask=60;                                                   % Preventive Tasks = Annual services 
    TypeTimes = [TType1 TType2 TType3 TType4 TType5 TPrevTask];            % Array of the different maintenance types 


    % Parameters for generating preventive tasks                                                               

    AmountPrevScheduled = 0.7;                                      % Amount of preventive maintenance demand that are scheduled for the planning period
    AmountPrevGenerated = 1.5;                                      % How many extra preventive tasks than the demand that are created for the planning period
    AmountPrev = [0.5 0.25 0 0.25];                                 % Percentage of preventive tasks to be performed during summer(1), fall(2), winter(3) and spring(4) 


    % Data to calculate costs: 

    ElPrice = [90 90 90 90];                                        % Electricity price in /MWh including subsidies for summer(1), fall(2), winter(3) and spring(4) 
    Elsertificate = 0;                                              % Optional subsidy scheme in EU/MWh
    ElRevenue = ElPrice + Elsertificate;                            % The total price from selling 1 MWh

    CostStayOut = 0;                                                % Costs for an AV to stay out of the depot between two shifts 

                                                                    % Turbine effiency (in percentage of maximum output) for wind speeds ranging from 0 to 30m/s:
    TurbineEfficiency = [0 0 0 0 0.03 0.03 0.04 0.07 0.12 0.20 0.26 0.35 0.53 0.87 0.95 1 1 1 1 1 1 1 1 1 0 0 0 0 0 0 0];

    alphaLowWind = 0.25;                                            % The percentage of maximum power output used to calculate the punishment cost of not performing extra preventive tasks if wind speeds are low/Upper bound for the efficiency when you want to start doing extra preventive tasks


   
% -------------------------------------------------------------------------------------------------
% GENERATING TASKS 
% -------------------------------------------------------------------------------------------------

% Data for generatting preventive tasks in the simulator: 

    YearlyPreventiveTasks = zeros(1,nWindFarms);                                      % Calculating total number of yearly tasks in all wind farms
       for i = 1:nWindFarms
           YearlyPreventiveTasks(i) = nTurbines(i); % One preventive task for each turbine
       end % for i    

   
% Corrective tasks 

    NumberOfTasks = 0; 

    % Probabilities of tasks occuring in one shift 
    PAlarmToTask = 0.9;                 % The probability of an alarm resulting in a task (the amount of alarms that are not false)
    PType2 = (7.5/365);
    PType3 = (3/365);
    PType4 = (0.275/365);
    PType5 = (0.04/365);

    PType2345 = PType2+PType3+PType4+PType5; % To calculate PFalse
    PType12345 = PType2345/PAlarmToTask;
    PFalse = PType12345*(1-PAlarmToTask);
    PType1 = PFalse+PType2+PType3+PType4+PType5;
    PNoCorr0 = (1-(PType2+PType3+PType4+PType5));
    PNoCorr = (1-PType1);


   % For shift one both alarms and "old discovered" tasks are generated. These are represented by tasks for shift "zero":  
    for shift = 1:1
        nCorrTasks = 0;  
        nTasks = 0; 
        VesselsTasks = [];
        WindFarmTasks = []; 
        TTask = []; 
        TechTask = []; 
        Type = [];
        TasksWindFarm = [];
        ExtraOrdinaryTask = [];

        % generating tasks: 
        c=1;
        k=[];
        InPark = zeros(1, nWindFarms); 
        for i=1:nWindFarms
            k(i)=1;
            for p=1:nTurbines(i)
                x = sum(rand >= cumsum([PNoCorr0, PType2, PType3, PType4, PType5]) );
                if x>0
                    WindFarmTasks(i,k(i)) = c;
                    k(i)=k(i)+1;
                    InPark(i) = 1;
                   if x==1
                         TTask(c)=TType2;
                         Type(c)=2;
                         TechTask(c)= TechTask2;
                    elseif x==2
                         TTask(c)=TType3;
                         Type(c)=3;
                         TechTask(c)= TechTask3;
                    elseif x==3
                         TTask(c)=TType4;
                         Type(c)=4;
                         TechTask(c)= TechTask4;
                    elseif x==4
                         TTask(c)=TType5;
                         Type(c)=5;
                         TechTask(c)= TechTask5;
                    end
                    c=c+1;
                    nCorrTasks=nCorrTasks+1;
                end
            end
        end

        % Adjusting for the case of only tasks in one wind farm: 
        if InPark(2) == 0
          WindFarmTasks(2,:) = 0;
        end % if 

            nTasks = nCorrTasks;                          % Total number of tasks

        % TasksWindFarm(m):         
        % A vector with which wind farm each tasks are located in 
        if nTasks > 0 
            TasksWindFarm = zeros(1,nTasks);             
            for m = 1:nTasks
                for i = 1:nWindFarms
                    for j = 1:length(WindFarmTasks(i,:))
                        if m == WindFarmTasks(i,j)
                            TasksWindFarm(m) = i;
                        end % if 
                    end % for j 
                end % if i 
            end % m
        
            
     % VesselsTasks(m,v) 
     % VesselsTasks gives a list of which vessels can perform which maintenace
     % types. VesselsTasks(m,v) = 1 if vessel v can perform task m, 0 otherwise
       
        for t=1:(nTasks)
            if  Type(t) == 2 || Type(t) == 3 || Type(t) == 4 || Type(t) == 6
                VesselsTasks(t,1)=1;
                VesselsTasks(t,2)=1;
                VesselsTasks(t,3)=1;
            elseif Type(t) == 5
                VesselsTasks(t,1)=1;
                VesselsTasks(t,2)=0;
                VesselsTasks((t),3)=0;      
            end
        end
        

        % ExtraOrdinaryTask(m)
        % ExtraOrdinaryTask =1 if task m requires vessel to stay by the
        % turbine while being performed
        for m = 1:nTasks
              if Type(m) == 2
                ExtraOrdinaryTask(m) = 0;
            elseif Type(m) == 3
                ExtraOrdinaryTask(m) = 0;
            elseif Type(m) == 4
                ExtraOrdinaryTask(m) = 0;
            elseif Type(m) == 5
               ExtraOrdinaryTask(m) = 1;
            elseif Type(m) == 6
               ExtraOrdinaryTask(m) = 0;
            end     
        end % for m 
        
        % Updating WindFarmTasks so that task numbers are correct (not overlapping previous tasks)

        for i=1:nWindFarms
            for m = 1:size(WindFarmTasks,2)
                if WindFarmTasks(i,m) > 0
                    WindFarmTasks(i,m) = WindFarmTasks(i,m) + NumberOfTasks; 
                end % if 
            end % for m
        end % for i 
        WindFarmTasks;
   
        NumberOfTasks = NumberOfTasks + nTasks;
        
        % AlarmResultingInTask = 1 if task m is an alarm and results in a new corrective
        % task, 0 if alarm m is a false alarm, or task m is another type of corrective task 
        AlarmResultingInTask = zeros(1,nTasks);
        for m = 1:nTasks
            if Type(m) == 1
                if rand(1) <= PAlarmToTask
                    AlarmResultingInTask(m) = 1; 
                end % if 
            end % if 
        end % for 
        
    end % nTasks > 0 
    
    eval(['nCorrTasks_' num2str(shift-1) '=nCorrTasks;'])
    eval(['AlarmResultingInTask_' num2str(shift-1) '=AlarmResultingInTask;'])
    eval(['nTasks_' num2str(shift-1) '=nTasks;'])
    eval(['VesselsTasks_' num2str(shift-1) '=VesselsTasks;']) 
    eval(['WindFarmTasks_' num2str(shift-1) '=WindFarmTasks;']) 
    eval(['TTask_' num2str(shift-1) '=TTask;']) 
    eval(['TechTask_' num2str(shift-1) '=TechTask;']) 
    eval(['Type_' num2str(shift-1) '=Type;']) 
    eval(['TasksWindFarm_' num2str(shift-1) '=TasksWindFarm;']) 
    eval(['ExtraOrdinaryTask_' num2str(shift-1) '=ExtraOrdinaryTask;']) 
    eval(['NumberOfTasks_' num2str(shift-1) '=NumberOfTasks;'])
    
end % for shift = first shift


% Generating tasks for shifts > shift "zero":
% For shifts after the first only alarm tasks are generated:
for shift = 1:nShifts
        nCorrTasks = 0;  
        nTasks = 0; 
        VesselsTasks = [];
        WindFarmTasks = []; 
        TTask = []; 
        TechTask = []; 
        Type = [];
        TasksWindFarm = [];
        ExtraOrdinaryTask = [];
    
    
        c=1;
        k=[];
        InPark = zeros(1, nWindFarms); 
        for i=1:nWindFarms
            k(i)=1;
            for p=1:nTurbines(i)
                x = sum(rand >= cumsum([PNoCorr, PType1]) );
                if x>0
                    WindFarmTasks(i,k(i)) = c;
                    k(i)=k(i)+1;
                    InPark(i) = 1;
                    if x==1
                        TTask(c)=TType1;
                        Type(c)=1;
                        TechTask(c)= TechTask1;
                    end
                    c=c+1;
                    nCorrTasks=nCorrTasks+1;
                end
            end
        end

        % Adjusting for the case of only tasks in one wind farm: 
        if InPark(2) == 0
          WindFarmTasks(2,:) = 0;
        end % if 

        nTasks = nCorrTasks;                          % Total number of tasks

        if nTasks > 0 
            % TasksWindFarm(m): 
            TasksWindFarm = zeros(1,nTasks);             
            for m = 1:nTasks
                for i = 1:nWindFarms
                    for j = 1:length(WindFarmTasks(i,:))
                        if m == WindFarmTasks(i,j)
                            TasksWindFarm(m) = i;
                        end % if 
                    end % for j 
                end % if i 
            end % m

            % VesselsTasks(m,v) 

            for t=1:(nTasks)
                    VesselsTasks(t,1)=1;
                    VesselsTasks(t,2)=1;
                    VesselsTasks(t,3)=1;
            end

            % ExtraOrdinaryTask(m)
            for m = 1:nTasks
                    ExtraOrdinaryTask(m) = 0;
            end % for m 

            % Updating WindFarmTasks so that task numbers are correct (not overlapping previous tasks)
            for i=1:nWindFarms
                for m = 1:size(WindFarmTasks,2)
                    if WindFarmTasks(i,m) > 0
                        WindFarmTasks(i,m) = WindFarmTasks(i,m) + NumberOfTasks; 
                    end % if 
                end % for m
            end % for i 

            NumberOfTasks = NumberOfTasks + nTasks;

            AlarmResultingInTask = zeros(1,nTasks);
            for m = 1:nTasks
                if Type(m) == 1
                    if rand(1) <= PAlarmToTask
                        AlarmResultingInTask(m) = 1; 
                    end % if 
                end % if 
            end % for m
        
        end % nTasks > 0 

        eval(['nCorrTasks_' num2str(shift) '=nCorrTasks;'])
        eval(['AlarmResultingInTask_' num2str(shift) '=AlarmResultingInTask;'])
        eval(['nTasks_' num2str(shift) '=nTasks;'])
        eval(['VesselsTasks_' num2str(shift) '=VesselsTasks;']) 
        eval(['WindFarmTasks_' num2str(shift) '=WindFarmTasks;']) 
        eval(['TTask_' num2str(shift) '=TTask;']) 
        eval(['TechTask_' num2str(shift) '=TechTask;']) 
        eval(['Type_' num2str(shift) '=Type;']) 
        eval(['TasksWindFarm_' num2str(shift) '=TasksWindFarm;']) 
        eval(['ExtraOrdinaryTask_' num2str(shift) '=ExtraOrdinaryTask;']) 
        eval(['NumberOfTasks_' num2str(shift) '=NumberOfTasks;'])
    
end % for shift = 2:nShifts

% -------------------------------------------------------------------------------------------------
% GENERATING WEATHER 
% -------------------------------------------------------------------------------------------------
% Creating weather window based on data from wind_FINO.txt

    % Choosing random week to load from data file 

    year = randi([1,9]);                                        % Year 1 = 2004,..., Year 9 = 2012

    if Season == 1                                   % Summer
        LowerBoundRow = (year-1)*365*24 + (31 + 28 + 31 + 30 + 31 +1)*24;
        UpperBoundRow = (year-1)*365*24 + (31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 - 7)*24;

    elseif Season == 2                               % Fall 
        LowerBoundRow = (year-1)*365*24 + (31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 + 1)*24;
        UpperBoundRow = (year-1)*365*24 + (31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 + 30 + 31 + 30 - 7)*24;

    elseif Season == 3                               % Winter 
        if year == 1                                            % If year = 2004, cannot include december 2013
        LowerBoundRow = (year-1)*365*24 + (1)*24;
        else 
        LowerBoundRow = (year-1)*365*24 - (31)*24;
        end 
        UpperBoundRow = (year-1)*365*24 + (31 + 28 - 7)*24; 

    elseif Season == 4                               % Spring
        LowerBoundRow = (year-1)*365*24 + (31 + 28 +1)*24;
        UpperBoundRow = (year-1)*365*24 + (31 + 28 + 31 + 30 + 31 - 7)*24;
    end % if 


    LongWindHorizonCounter = nShifts;                         % Storing weather data for the entire simulation period       
    FirstRow = randi([LowerBoundRow,UpperBoundRow]);          % Row of first hour in first day
    LastRow = FirstRow + LongWindHorizonCounter*24;           % Row of last hour in last day 
    LengthWeatherTable = LastRow-FirstRow;


    %Import datafile 
    Data = load('wind_FINO.txt');

    % Create weather table
    Weather = zeros(LengthWeatherTable,8);

    for i = 1:LengthWeatherTable
        Weather(i,:) = Data((i+FirstRow),:);
    end % for i 

 
    % Create a table of the rows from first to last called Weather(hour,data)
    WaveLimit = [WaveLimitAV WaveLimitSES WaveLimitCTV];
    StartWindow = [];
    EndWindow = [];
    L = [];
    U = [];

    WindLongHorizon = [];

    % Finding the time window of each day
    for s = 1:LongWindHorizonCounter                                          
        sumwind(s) = 0;

        teller = zeros(1,length(WaveLimit));
        diff = zeros(1,length(WaveLimit)); 

    for h = 7:18                                             % Only interested in the hours in the shift (1-12)
        sumwind(s) = sumwind(s)+ Weather((s-1)*24+h,7);
        WindLongHorizon(s) = round(sumwind(s)/12);

        for i = 1:length(WaveLimit)
            if h == 7 && Weather((s-1)*24+h,8) < WaveLimit(i)  
                 teller(1,i) = teller(1,i)+1;
                 L(i,s,teller(1,i)) = h-7;
            elseif h >= 8 && h<18 && Weather((s-1)*24+h,8)< WaveLimit(i) && Weather((s-1)*24+h-1,8) >= WaveLimit(i)
                 teller(1,i) = teller(1,i)+1;
                 L(i,s,teller(1,i)) = h-7;
            elseif h >= 8 && h < 18 && Weather((s-1)*24+h,8)>= WaveLimit(i) && Weather((s-1)*24+h-1,8) < WaveLimit(i)
                 U(i,s,teller(1,i)) = h-8;
            elseif h == 18 && Weather((s-1)*24+h-1,8) < WaveLimit(i)
                 U(i,s,teller(1,i)) = h-6;
            elseif h == 18 && teller(i) == 0
                 teller(1,i) = 1;
                 L(i,s,teller(1,i)) = h-6;
                 U(i,s,teller(1,i)) = h-6;
            end % if CTV
        end % for i   
    end % for h 
    
        for i =1:length(WaveLimit)
            for j = 1:teller(1,i)
                if (U(i,s,j)-L(i,s,j))>diff(i)
                    diff(i) = U(i,s,j)-L(i,s,j);
                    StartWindow(i,s) = L(i,s,j);
                    EndWindow(i,s) = U(i,s,j);
                end % if 
            end % for j
            if diff(i) == 0
                StartWindow(i,s) = nTShift;
                EndWindow(i,s) = nTShift;
            end       
        end % for i 
    end % for s 
    
    % Creating a dummy variable with weather window for each vessel 
    for v = 1:nAV
        StartWindowDummy(v,:) = StartWindow(1,:);
        EndWindowDummy(v,:) = EndWindow(1,:);
    end % for v
    for v = nAV+1:nAV+nSES
        StartWindowDummy(v,:) = StartWindow(2,:);
        EndWindowDummy(v,:) = EndWindow(2,:);
    end % for v
    for v = nAV+nSES+1:nVessels
        StartWindowDummy(v,:) = StartWindow(3,:);
        EndWindowDummy(v,:) = EndWindow(3,:);
    end % for v
   
    % Creating weather arrays: 
    StartWindow = [];
    StartWindow = StartWindowDummy;
    EndWindow = [];
    EndWindow = EndWindowDummy;
    
    
% -------------------------------------------------------------------------------------------------
% GENERATING TASK AND SHIFT INDEPENDENT SETS AND PARAMETERS
% -------------------------------------------------------------------------------------------------

% Distance Matrix(i,j)
% Distance matrix gives distances between every node i and j, including both wind
% farms and the depot
    Distance = zeros(nWindFarms+2);
    for i = 1:(nWindFarms+2)
        for j = 1:(nWindFarms+2)
        if i==j
            Distance(i,j) = 0; 
        elseif i == 1 && j > 1 && j<(nWindFarms+2)
            Distance(i,j) = DistanceDepot(j-1);
        elseif j ==1 && i>1 && i<(nWindFarms+2)
            Distance(i,j) = DistanceDepot(i-1);
        elseif i > 1 && i<(nWindFarms+2) && j <(nWindFarms+2) && j>1
            Distance(i,j) = DistanceFarms((i-1),(j-1));
        elseif j ==(nWindFarms+2) && i>1 && i<(nWindFarms+2)
            Distance(i,j) = DistanceDepot(i-1);
        end % if 
        end % for j 
    end % for i 
    Distance((nWindFarms+2),:) = Distance(1,:);

% CTrans(i,j,v)
% CTrans gives the traveling costs for each vessel v between every node i and j, including both wind
% farms and the depot
    CTrans = [];    
    for i = 1:(nWindFarms+2)
        for j = 1:(nWindFarms+2)
            for v = 1:nVessels
                   if v <= nAV
                    CTrans(i,j,v) = (Distance(i,j)/SpeedAV)*CostAV;
                   elseif v <= nAV+nSES
                    CTrans(i,j,v) = (Distance(i,j)/SpeedSES)*CostSES;
                   else 
                    CTrans(i,j,v) = (Distance(i,j)/SpeedCTV)*CostCTV;
                   end % if
            end % for v
       end % for j
    end % for i 

% TTrans(i,j,v)
% TTrans gives the traveling times for each vessel v between every node i and j, including both wind
% farms and the depot
    TTrans = [];
    for i = 1:(nWindFarms+2)
        for j = 1:(nWindFarms+2)
            for v = 1:nVessels
                   if v <= nAV
                    TTrans(i,j,v) = Distance(i,j)/SpeedAV;
                   elseif v <= nAV+nSES
                    TTrans(i,j,v) = Distance(i,j)/SpeedSES;
                   else
                    TTrans(i,j,v) = Distance(i,j)/SpeedCTV;  
                   end % if
            end % for v
       end % for j
    end % for i
    

% IntTransport(i,v)
% Average internal transfer time for vessel v between turbines in wind farm i
    IntTransport = [];
        for i = 1:nWindFarms
            for v=1:nAV
                IntTransport(i,v) = IntAV(i);
            end % for v
            for v=nAV+1:nAV+nSES
                IntTransport(i,v) = IntSES(i);
            end % for v
            for v = nAV+nSES+1:nVessels
                IntTransport(i,v) = IntCTV(i);
            end % for v
        end % for i 

% CIntTrans(v)
% Cost to travel within a park 
    CIntTrans = [StayOutCostAV, StayOutCostSES, StayOutCostCTV];


% TShifts(s)
% An array of the length of every shift s
    TShift = ones(nShifts, 1) * nTShift;
    
% PStart(i,v)
% PStart is a vector consisting of the location of the AVs v when the
% planning horixon starts. If PStart(j,v) = 1 then vessel v starts in node
% j, 0 otherwise

    PStart =zeros((nWindFarms+2), nAV);
    for v = 1:nAV
    i = randi([2, (nWindFarms+2)]);
    PStart(i,v) = 1;
    end

% DStart(v)
% DStart is a vector consisting of the number of shifts an AV v has been
% out of the depot when the planning period starts 

    DStartL = 1;                                            % Lower bound on how many days an AV has already been out when the time period starts
    DStart = [];
    for v = 1:nAV
        if PStart(nWindFarms+2,v)==1
            DStart(v) = 0;
        else
            DStart(v) = randi([DStartL, (DLimit-1)]);
        end
    end
    
% NigthCost(v)
% A vector consisting of the costs for an AV to stay out of the depot
% between two shifts 

    NightCostOut = ones(1,nAV)*CostStayOut;
    
% NoPrevPunish
% The punishment of not performing a scheduled preventive tasks 

    LargestTurbineEffect = 0;                                   % Calculating for the turbines with largest effect
    for i = 1: size(WindFarmTurbineSize)
        if LargestTurbineEffect<WindFarmTurbineSize(i)
            LargestTurbineEffect = WindFarmTurbineSize(i);
        end % if
    end % for 

    LongestFarmDistance = 0;                                    % Finding the farm furthest from shore
    for i = 1:size(DistanceDepot)
        if LongestFarmDistance < DistanceDepot(i)
            LongestFarmDistance = i;
        end % if
    end % for i 

    %NoPrevPunish = floor(ElRevenue(Season)*LargestTurbineEffect*(nTShift+1) + CTrans(1,(LongestFarmDistance+1),1)+ CIntTrans(1));  %NoPrevPunish = the largest CLoss + the largest transport cost possible
    %NoPrevPunish = floor(ElRevenue(Season)*LargestTurbineEffect*(nTShift+1) + CTrans(1,(LongestFarmDistance+1),1)+ CIntTrans(1));  %NoPrevPunish = the largest CLoss + the largest transport cost possible
    NoPrevPunish = floor(ElRevenue(Season)*LargestTurbineEffect*(nTShift+1) + ((Distance(1,(LongestFarmDistance+1))/SpeedAV)*CostAV)+ CIntTrans(1)); 

% TechCap(v)
% The technician capacity of vessel v
    TechCap = [];

    for v = 1:nAV
        TechCap(v) = CapAV;
    end % for v 
    for v = nAV+1:nAV+nSES
        TechCap(v) = CapSES;
    end % for v 
    for v = nAV+nSES+1:nVessels
        TechCap(v) = CapCTV; 
    end % for v 


%----------------------------------------------------------------------
% UPDATING VARIABLES SO THAT THEY INCLUDE ALL SHIFTS 
%----------------------------------------------------------------------

    nCorrTasks = nCorrTasks_0;  
    nTasks = nTasks_0; 
    VesselsTasks = VesselsTasks_0;
    WindFarmTasks = WindFarmTasks_0; 
    TTask = TTask_0; 
    TechTask = TechTask_0; 
    Type = Type_0;
    TasksWindFarm = TasksWindFarm_0;
    ExtraOrdinaryTask = ExtraOrdinaryTask_0;
    NumberOfTasks = nTasks_0; 
    AlarmResultingInTask = AlarmResultingInTask_0; 

    for period = 1:nShifts
        NewWindFarmTasks = eval(['WindFarmTasks_' num2str(period)]);          
        NewVesselsTasks = eval(['VesselsTasks_' num2str(period)]);
        NewType = eval(['Type_' num2str(period)]);
        NewTTask = eval(['TTask_' num2str(period)]);
        NewTechTask = eval(['TechTask_' num2str(period)]);
        NewnCorrTasks = eval(['nCorrTasks_' num2str(period)]);
        NewTasksWindFarm = eval(['TasksWindFarm_' num2str(period)]);
        NewExtraOrdinaryTask = eval(['ExtraOrdinaryTask_' num2str(period)]);
        NewnTasks = NewnCorrTasks;
        NewAlarmResultingInTask = eval(['AlarmResultingInTask_' num2str(period)]);

        for m = 1:(NewnTasks)
        TTask(m+NumberOfTasks) = NewTTask(m);
        VesselsTasks(m+NumberOfTasks,:) = NewVesselsTasks(m,:); 
        TechTask(m+NumberOfTasks) = NewTechTask(m);
        ExtraOrdinaryTask(m+NumberOfTasks) = NewExtraOrdinaryTask(m);
        Type(m+NumberOfTasks) = NewType(m);
        TasksWindFarm(m+NumberOfTasks) = NewTasksWindFarm(m);
        AlarmResultingInTask(m+NumberOfTasks) = NewAlarmResultingInTask(m);
        end % for m 

        nTasks = nTasks + NewnTasks;
        NumberOfTasks = NumberOfTasks + NewnTasks; 
        nCorrTasks = nCorrTasks + NewnCorrTasks;


        % WindFarmTasks(i,m) 
        A = size(WindFarmTasks,2);                     
        B = size(NewWindFarmTasks,2);                  

        for i = 1:B                                     
            WindFarmTasks(:,A+i) = 0;   
        end % for i 

        for i = 1:nWindFarms                            
        t = 1; 
            for m = 1:size(WindFarmTasks,2)
                if t <= B 
                    if WindFarmTasks(i,m) == 0
                        WindFarmTasks(i,m) = NewWindFarmTasks(i,t); 
                        t = t+1; 
                    end % if 
                end % if 
            end % for m 
        end % for i 

        C = size(WindFarmTasks,2)+1;                          
        for m = 1:C-1                                        
            t=C-m;
            if WindFarmTasks(:,t) == 0
                WindFarmTasks(:,t) = []; 
            end % if 
        end % for m 
        WindFarmTasks;

    end % period = 2:nShifts




% Giving properties to the tasks that occur after checking an alarm:  
    PAlarm2 = PType2/(PType2+PType3+PType4+PType5);
    PAlarm3 = PType3/(PType2+PType3+PType4+PType5);
    PAlarm4 = PType4/(PType2+PType3+PType4+PType5);
    PAlarm5 = PType5/(PType2+PType3+PType4+PType5);

    alarmtask = 0; 
    for m = 1:nTasks
        if AlarmResultingInTask(m) == 1 
           alarmtask = alarmtask+1; 
           NewTask(alarmtask) = 1;
           x = sum(rand >= cumsum([PAlarm2, PAlarm3, PAlarm4, PAlarm5]) );
           if x == 0
               AlarmType(alarmtask) = 2; 
               AlarmTechTask(alarmtask) = TechTask2; 
               AlarmTTask(alarmtask) = TType2; 
               AlarmExtraOrdinaryTask(alarmtask) = 0; 
               AlarmTasksWindFarm(alarmtask) = TasksWindFarm(m);
           elseif x == 1
               AlarmType(alarmtask) = 3; 
               AlarmTechTask(alarmtask) = TechTask3; 
               AlarmTTask(alarmtask) = TType3; 
               AlarmExtraOrdinaryTask(alarmtask) = 0;
               AlarmTasksWindFarm(alarmtask) = TasksWindFarm(m); 
           elseif x == 2
               AlarmType(alarmtask) = 4; 
               AlarmTechTask(alarmtask) = TechTask4; 
               AlarmTTask(alarmtask) = TType4; 
               AlarmExtraOrdinaryTask(alarmtask) = 0; 
               AlarmTasksWindFarm(alarmtask) = TasksWindFarm(m); 
           elseif x == 3
               AlarmType(alarmtask) = 5; 
               AlarmTechTask(alarmtask) = TechTask5; 
               AlarmTTask(alarmtask) = TType5; 
               AlarmExtraOrdinaryTask(alarmtask) = 1; 
               AlarmTasksWindFarm(alarmtask) = TasksWindFarm(m); 
           end    
        end % if 
    end % for m 

    for m = 1:alarmtask
        if AlarmType(m) == 2 || AlarmType(m) == 3 || AlarmType(m) == 4 
            AlarmVesselsTasks(m,1)=1;
            AlarmVesselsTasks(m,2)=1;
            AlarmVesselsTasks(m,3)=1;
        elseif AlarmType(m) == 5
            AlarmVesselsTasks(m,1)=1;
            AlarmVesselsTasks(m,2)=0;
            AlarmVesselsTasks(m,3)=0;
        end 
    end % m 

    StartWindow;
    EndWindow;
    nTasks_0+nTasks_1+nTasks_2+nTasks_3;
    Type_0;
    PStart;

%----------------------------------------------------------------------
% SAVING VARIABLES TO USE IN SIMULATOR
%----------------------------------------------------------------------

    save(name)